<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Error - Client Error</title>
    <style>
        body {
            font-family: Arial, sans-serif;
            background-color: #f4f4f4;
            color: #333;
            text-align: center;
            padding: 50px;
        }
        h1 {
            font-size: 3em;
            color: #e74c3c;
        }
        h2 {
            font-size: 2em;
            color: #555;
        }
        ul {
            list-style-type: none;
            padding: 0;
        }
        li {
            font-size: 1.2em;
            margin: 10px 0;
        }
    </style>
</head>
<body>
    <h1>Error {{ status_code }}</h1>
    <h2>Oops! Something went wrong.</h2>
    <p>We encountered an error processing your request. Here are the details:</p>
    <ul>
        <li><strong>Status Code:</strong> {{ status_code }} - {{ status }}: {{ description }}</li>
    </ul>

{% if reasons %}
    <p>Additional information include:</p>
    <ul>
        {% for reason in reasons %}
            <li>{% if reason.status_code %}<strong>Error {{ reason.status_code }}:</strong>{% endif %} {{ reason.message | e }}</li>
        {% endfor %}
    </ul>
    {% else %}
    <p>No additional reasons were provided for this error.</p>
{% endif %}
    <p>Please check the URL or contact support if the problem persists.</p>
</body>
</html>